package main

import (
	"strings"

	"github.com/spf13/viper"
	"gopkg.in/macaron.v1"
)

func mqttPage(ctx *macaron.Context) {
	ctx.Data["schema"] = viper.GetString("MqttSchema")
	ctx.Data["host"] = viper.GetString("MqttHost")
	ctx.Data["port"] = viper.GetString("MqttPort")
	ctx.Data["clientid"] = viper.GetString("MqttCliendID")
	ctx.Data["prefix"] = viper.GetString("MqttPrefix")
	ctx.Data["username"] = viper.GetString("MqttUsername")
	ctx.Data["password"] = viper.GetString("MqttPassword")

	ctx.Data["schemas"] = []string{"tcp", "ssl", "ws"}
	ctx.HTML(200, "mqtt")
}

type MQTTPostForm struct {
	Schema   string `form:"schema" binding:"Required"`
	Host     string `form:"host" binding:"Required"`
	Port     string `form:"port" binding:"Required"`
	Prefix   string `form:"prefix" binding:"Required"`
	ClientID string `form:"clientid" binding:"Required"`
	Username string `form:"username"`
	Password string `form:"password"`
}

func mqttPost(ctx *macaron.Context, f MQTTPostForm) {
	schema := strings.ToLower(strings.TrimSpace(f.Schema))
	switch schema {
	case
		"tcp",
		"ssl",
		"ws":
		viper.Set("MqttSchema", schema)
	default:
		mqttPage(ctx)
		return
	}

	viper.Set("MqttHost", strings.ToLower(strings.TrimSpace(f.Host)))
	viper.Set("MqttPort", strings.ToLower(strings.TrimSpace(f.Port)))
	viper.Set("MqttCliendID", strings.TrimSpace(f.ClientID))
	viper.Set("MqttPrefix", strings.TrimSpace(f.Prefix))
	viper.Set("MqttUsername", strings.TrimSpace(f.Username))
	viper.Set("MqttPassword", f.Password)

	viper.WriteConfig()

	mqttPage(ctx)
}
